@@ -30,6 +30,16 @@
 #include "srm_util.h"
 #include "gfal_srm_ifce_internal.h"
 
+/* Not defined on kfreebsd */
+#ifndef ETIME
+#define ETIME EIO
+#endif
+
+/* Not defined on kfreebsd and hurd */
+#ifndef EBADR
+#define EBADR EIO
+#endif
+
 #ifdef CMAKE_BUILD
 #define namespaces_srmv2 namespaces
 #endif
@@ -135,6 +145,37 @@ void set_estimated_wait_time(struct srm_internal_context *internal_context, int
 }
 
 
+static void srm_send_one_header(GQuark key_id, gpointer data, gpointer user_data)
+{
+    struct soap* soap = (struct soap*)user_data;
+    struct srm_context* c = (struct srm_context*)soap->user;
+    c->ext->original_fposthdr(soap, g_quark_to_string(key_id), data);
+}
+
+
+static int srm_post_header(struct soap* soap, const char* key, const char* value)
+{
+    struct srm_context* c = (struct srm_context*)soap->user;
+    assert(c->ext->original_fposthdr);
+
+    // If NULL, NULL, it is our chance to push our headers, since the caller
+    // expects just to end the headers with a \n\r now
+    if (key == NULL && value == NULL) {
+        g_datalist_foreach(&c->ext->additional_headers, srm_send_one_header, soap);
+    }
+    // Override user-agent
+    else if (c->ext->user_agent[0] && strncasecmp(key, "User-Agent", 10) == 0) {
+        char* buffer = g_strdup_printf("%s srm-ifce/%s %s", c->ext->user_agent, VERSION, value);
+        int r = c->ext->original_fposthdr(soap, key, buffer);
+        g_free(buffer);
+        return r;
+    }
+
+    // Forward the request
+    return c->ext->original_fposthdr(soap, key, value);
+}
+
+
 void srm_context_soap_init(struct srm_context* c)
 {
     assert(c);
@@ -173,6 +214,13 @@ void srm_context_soap_init(struct srm_context* c)
     c->soap->recv_timeout= c->timeout_ops;
     c->soap->send_timeout = c->timeout_ops;
     c->soap->connect_timeout = c->timeout_conn;
+
+    // Allow to inject our own headers
+    if (c->ext) {
+        c->soap->user = c;
+        c->ext->original_fposthdr = c->soap->fposthdr;
+        c->soap->fposthdr = srm_post_header;
+    }
 }
 
 
@@ -232,6 +280,8 @@ int statuscode2errno (int statuscode)
 			return (EIDRM);
 		case SRM_USCOREABORTED:
 			return (ECANCELED);
+		case SRM_USCORETOO_USCOREMANY_USCORERESULTS:
+			return (EFBIG);
 		case SRM_USCORESUCCESS:
 		case SRM_USCOREFILE_USCOREPINNED:
 		case SRM_USCORESPACE_USCOREAVAILABLE:
@@ -1077,6 +1127,8 @@ int copy_mdfilestatuses(struct srm2__TReturnStatus *reqstatp,
 		if (repfs->pathDetailArray[i]->status)
 		{
 			(*statuses)[i].status = statuscode2errno(repfs->pathDetailArray[i]->status->statusCode);
+			if (reqstatp->statusCode == SRM_USCORETOO_USCOREMANY_USCORERESULTS)
+				(*statuses)[i].status = EFBIG;
 
 			if ((*statuses)[i].status) {
 				if (repfs->pathDetailArray[i]->status->explanation && repfs->pathDetailArray[i]->status->explanation[0])
